@@ -27,13 +27,13 @@ from flash.core.data.utils import (
     CurrentRunningStageContext,
 )
 
-if TYPE_CHECKING:  # pragma: no-cover
-    from flash.core.data.process import Preprocess
+if TYPE_CHECKING:
+    from flash.core.data.process import Deserializer, Preprocess, Serializer
 
 
 class _Sequential(torch.nn.Module):
-    """
-    This class is used to chain 3 functions together for the _Preprocessor ``per_sample_transform`` function.
+    """This class is used to chain 3 functions together for the _Preprocessor ``per_sample_transform`` function.
+
     1. ``pre_tensor_transform``
     2. ``to_tensor_transform``
     3. ``post_tensor_transform``
@@ -41,9 +41,9 @@ class _Sequential(torch.nn.Module):
 
     def __init__(
         self,
-        preprocess: 'Preprocess',
-        pre_tensor_transform: Callable,
-        to_tensor_transform: Callable,
+        preprocess: "Preprocess",
+        pre_tensor_transform: Optional[Callable],
+        to_tensor_transform: Optional[Callable],
         post_tensor_transform: Callable,
         stage: RunningStage,
         assert_contains_tensor: bool = False,
@@ -66,20 +66,22 @@ class _Sequential(torch.nn.Module):
         self.callback.on_load_sample(sample, self.stage)
 
         with self._current_stage_context:
-            with self._pre_tensor_transform_context:
-                sample = self.pre_tensor_transform(sample)
-                self.callback.on_pre_tensor_transform(sample, self.stage)
+            if self.pre_tensor_transform is not None:
+                with self._pre_tensor_transform_context:
+                    sample = self.pre_tensor_transform(sample)
+                    self.callback.on_pre_tensor_transform(sample, self.stage)
 
-            with self._to_tensor_transform_context:
-                sample = self.to_tensor_transform(sample)
-                self.callback.on_to_tensor_transform(sample, self.stage)
+            if self.to_tensor_transform is not None:
+                with self._to_tensor_transform_context:
+                    sample = self.to_tensor_transform(sample)
+                    self.callback.on_to_tensor_transform(sample, self.stage)
 
-            if self.assert_contains_tensor:
-                if not _contains_any_tensor(sample):
-                    raise MisconfigurationException(
-                        "When ``to_tensor_transform`` is overriden, "
-                        "``DataPipeline`` expects the outputs to be ``tensors``"
-                    )
+                if self.assert_contains_tensor:
+                    if not _contains_any_tensor(sample):
+                        raise MisconfigurationException(
+                            "When ``to_tensor_transform`` is overriden, "
+                            "``DataPipeline`` expects the outputs to be ``tensors``"
+                        )
 
             with self._post_tensor_transform_context:
                 sample = self.post_tensor_transform(sample)
@@ -98,30 +100,77 @@ class _Sequential(torch.nn.Module):
         )
 
 
+class _DeserializeProcessor(torch.nn.Module):
+    def __init__(
+        self,
+        deserializer: "Deserializer",
+        preprocess: "Preprocess",
+        pre_tensor_transform: Callable,
+        to_tensor_transform: Callable,
+    ):
+        super().__init__()
+        self.preprocess = preprocess
+        self.callback = ControlFlow(self.preprocess.callbacks)
+        self.deserializer = convert_to_modules(deserializer)
+        self.pre_tensor_transform = convert_to_modules(pre_tensor_transform)
+        self.to_tensor_transform = convert_to_modules(to_tensor_transform)
+
+        self._current_stage_context = CurrentRunningStageContext(RunningStage.PREDICTING, preprocess, reset=False)
+        self._pre_tensor_transform_context = CurrentFuncContext("pre_tensor_transform", preprocess)
+        self._to_tensor_transform_context = CurrentFuncContext("to_tensor_transform", preprocess)
+
+    def forward(self, sample: str):
+
+        sample = self.deserializer(sample)
+
+        with self._current_stage_context:
+            with self._pre_tensor_transform_context:
+                sample = self.pre_tensor_transform(sample)
+                self.callback.on_pre_tensor_transform(sample, RunningStage.PREDICTING)
+
+            with self._to_tensor_transform_context:
+                sample = self.to_tensor_transform(sample)
+                self.callback.on_to_tensor_transform(sample, RunningStage.PREDICTING)
+
+        return sample
+
+
+class _SerializeProcessor(torch.nn.Module):
+    def __init__(
+        self,
+        serializer: "Serializer",
+    ):
+        super().__init__()
+        self.serializer = convert_to_modules(serializer)
+
+    def forward(self, sample):
+        return self.serializer(sample)
+
+
 class _Preprocessor(torch.nn.Module):
     """
-        This class is used to encapsultate the following functions of a Preprocess Object:
-        Inside a worker:
-            per_sample_transform: Function to transform an individual sample
-                Inside a worker, it is actually make of 3 functions:
-                    * pre_tensor_transform
-                    * to_tensor_transform
-                    * post_tensor_transform
-            collate: Function to merge sample into a batch
-            per_batch_transform: Function to transform an individual batch
-                * per_batch_transform
+    This class is used to encapsultate the following functions of a Preprocess Object:
+    Inside a worker:
+        per_sample_transform: Function to transform an individual sample
+            Inside a worker, it is actually make of 3 functions:
+                * pre_tensor_transform
+                * to_tensor_transform
+                * post_tensor_transform
+        collate: Function to merge sample into a batch
+        per_batch_transform: Function to transform an individual batch
+            * per_batch_transform
 
-        Inside main process:
-            per_sample_transform: Function to transform an individual sample
-                * per_sample_transform_on_device
-            collate: Function to merge sample into a batch
-            per_batch_transform: Function to transform an individual batch
-                * per_batch_transform_on_device
+    Inside main process:
+        per_sample_transform: Function to transform an individual sample
+            * per_sample_transform_on_device
+        collate: Function to merge sample into a batch
+        per_batch_transform: Function to transform an individual batch
+            * per_batch_transform_on_device
     """
 
     def __init__(
         self,
-        preprocess: 'Preprocess',
+        preprocess: "Preprocess",
         collate_fn: Callable,
         per_sample_transform: Union[Callable, _Sequential],
         per_batch_transform: Callable,
@@ -145,8 +194,8 @@ class _Preprocessor(torch.nn.Module):
         self._collate_context = CurrentFuncContext("collate", preprocess)
         self._per_batch_transform_context = CurrentFuncContext(f"per_batch_transform{extension}", preprocess)
 
+    @staticmethod
     def _extract_metadata(
-        self,
         samples: List[Dict[str, Any]],
     ) -> Tuple[List[Dict[str, Any]], Optional[List[Dict[str, Any]]]]:
         metadata = [s.pop(DefaultDataKeys.METADATA, None) if isinstance(s, Mapping) else None for s in samples]
@@ -164,6 +213,10 @@ class _Preprocessor(torch.nn.Module):
             if self.apply_per_sample_transform:
                 with self._per_sample_transform_context:
                     _samples = []
+
+                    if isinstance(samples, Mapping):
+                        samples = [samples]
+
                     for sample in samples:
                         sample = self.per_sample_transform(sample)
                         if self.on_device:
@@ -174,8 +227,11 @@ class _Preprocessor(torch.nn.Module):
 
                 with self._collate_context:
                     samples, metadata = self._extract_metadata(samples)
-                    samples = self.collate_fn(samples)
-                    if metadata:
+                    try:
+                        samples = self.collate_fn(samples, metadata)
+                    except TypeError:
+                        samples = self.collate_fn(samples)
+                    if metadata and isinstance(samples, dict):
                         samples[DefaultDataKeys.METADATA] = metadata
                     self.callback.on_collate(samples, self.stage)
 
@@ -201,15 +257,16 @@ class _Preprocessor(torch.nn.Module):
 
 
 class _Postprocessor(torch.nn.Module):
-    """
-        This class is used to encapsultate the following functions of a Postprocess Object:
-        Inside main process:
-            per_batch_transform: Function to transform a batch
-            per_sample_transform: Function to transform an individual sample
-            uncollate_fn: Function to split a batch into samples
-            per_sample_transform: Function to transform an individual sample
-            save_fn: Function to save all data
-            save_per_sample: Function to save an individual sample
+    """This class is used to encapsultate the following functions of a Postprocess Object:
+
+    Inside main process:
+        per_batch_transform: Function to transform a batch
+        per_sample_transform: Function to transform an individual sample
+        uncollate_fn: Function to split a batch into samples
+        per_sample_transform: Function to transform an individual sample
+        save_fn: Function to save all data
+        save_per_sample: Function to save an individual sample
+        is_serving: Whether the Postprocessor is used in serving mode.
     """
 
     def __init__(
@@ -219,7 +276,8 @@ class _Postprocessor(torch.nn.Module):
         per_sample_transform: Callable,
         serializer: Optional[Callable],
         save_fn: Optional[Callable] = None,
-        save_per_sample: bool = False
+        save_per_sample: bool = False,
+        is_serving: bool = False,
     ):
         super().__init__()
         self.uncollate_fn = convert_to_modules(uncollate_fn)
@@ -228,11 +286,31 @@ class _Postprocessor(torch.nn.Module):
         self.serializer = convert_to_modules(serializer)
         self.save_fn = convert_to_modules(save_fn)
         self.save_per_sample = convert_to_modules(save_per_sample)
+        self.is_serving = is_serving
+
+    @staticmethod
+    def _extract_metadata(batch: Any) -> Tuple[Any, Optional[Any]]:
+        metadata = None
+        if isinstance(batch, Mapping) and DefaultDataKeys.METADATA in batch:
+            metadata = batch.pop(DefaultDataKeys.METADATA, None)
+        return batch, metadata
 
     def forward(self, batch: Sequence[Any]):
+        batch, metadata = self._extract_metadata(batch)
         uncollated = self.uncollate_fn(self.per_batch_transform(batch))
+        if metadata:
+            for sample, sample_metadata in zip(uncollated, metadata):
+                sample[DefaultDataKeys.METADATA] = sample_metadata
 
-        final_preds = type(uncollated)([self.serializer(self.per_sample_transform(sample)) for sample in uncollated])
+        final_preds = [self.per_sample_transform(sample) for sample in uncollated]
+
+        if self.serializer is not None:
+            final_preds = [self.serializer(sample) for sample in final_preds]
+
+        if isinstance(uncollated, Tensor) and isinstance(final_preds[0], Tensor):
+            final_preds = torch.stack(final_preds)
+        else:
+            final_preds = type(final_preds)(final_preds)
 
         if self.save_fn:
             if self.save_per_sample:
@@ -240,8 +318,7 @@ class _Postprocessor(torch.nn.Module):
                     self.save_fn(pred)
             else:
                 self.save_fn(final_preds)
-        else:
-            return final_preds
+        return final_preds
 
     def __str__(self) -> str:
         return (
@@ -256,7 +333,6 @@ class _Postprocessor(torch.nn.Module):
 def default_uncollate(batch: Any):
     """
     This function is used to uncollate a batch into samples.
-
     Examples:
         >>> a, b = default_uncollate(torch.rand((2,1)))
     """
@@ -268,13 +344,13 @@ def default_uncollate(batch: Any):
             return batch
         return list(torch.unbind(batch, 0))
 
-    elif isinstance(batch, Mapping):
+    if isinstance(batch, dict):
         return [batch_type(dict(zip(batch, default_uncollate(t)))) for t in zip(*batch.values())]
 
-    elif isinstance(batch, tuple) and hasattr(batch, '_fields'):  # namedtuple
-        return [batch_type(*default_uncollate(sample)) for sample in zip(*batch)]
+    if isinstance(batch, tuple) and hasattr(batch, "_fields"):  # namedtuple
+        return [batch_type(*sample) for sample in zip(*batch)]
 
-    elif isinstance(batch, Sequence) and not isinstance(batch, str):
-        return [default_uncollate(sample) for sample in batch]
+    if isinstance(batch, Sequence) and not isinstance(batch, str):
+        return [sample for sample in batch]
 
     return batch
