@@ -6,8 +6,7 @@ from torch.utils.data import Dataset
 
 
 class SplitDataset(Dataset):
-    """
-    SplitDataset is used to create Dataset Subset using indices.
+    """SplitDataset is used to create Dataset Subset using indices.
 
     Args:
 
@@ -20,12 +19,13 @@ class SplitDataset(Dataset):
         split_ds = SplitDataset(dataset, indices=[10, 14, 25])
 
         split_ds = SplitDataset(dataset, indices=[10, 10, 10, 14, 25], use_duplicated_indices=True)
-
     """
 
     _INTERNAL_KEYS = ("dataset", "indices", "data")
 
-    def __init__(self, dataset: Any, indices: List[int] = [], use_duplicated_indices: bool = False) -> None:
+    def __init__(self, dataset: Any, indices: List[int] = None, use_duplicated_indices: bool = False) -> None:
+        if indices is None:
+            indices = []
         if not isinstance(indices, list):
             raise MisconfigurationException("indices should be a list")
 
