@@ -15,7 +15,7 @@ import inspect
 import warnings
 from argparse import ArgumentParser, Namespace
 from functools import wraps
-from typing import Callable, List, Optional, Union
+from typing import Callable, Dict, List, Optional, Tuple, Union
 
 import torch
 from pytorch_lightning import LightningDataModule, LightningModule
@@ -28,11 +28,12 @@ from torch.utils.data import DataLoader
 
 import flash
 from flash.core.finetuning import _DEFAULTS_FINETUNE_STRATEGIES, instantiate_default_finetuning_callbacks
+from flash.core.utilities.imports import _SERVE_AVAILABLE
 
 
 def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs):
-    """Modified version of ``pytorch_lightning.utilities.argparse.from_argparse_args`` which populates ``valid_kwargs``
-    from ``pytorch_lightning.Trainer``."""
+    """Modified version of :func:`pytorch_lightning.utilities.argparse.from_argparse_args` which populates
+    ``valid_kwargs`` from :class:`pytorch_lightning.Trainer`."""
     if isinstance(args, ArgumentParser):
         args = cls.parse_argparser(args)
 
@@ -40,15 +41,17 @@ def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs):
 
     # we only want to pass in valid PLTrainer args, the rest may be user specific
     valid_kwargs = inspect.signature(PlTrainer.__init__).parameters
-    trainer_kwargs = dict((name, params[name]) for name in valid_kwargs if name in params)
+    trainer_kwargs = {name: params[name] for name in valid_kwargs if name in params}
     trainer_kwargs.update(**kwargs)
 
     return cls(**trainer_kwargs)
 
 
 def _defaults_from_env_vars(fn: Callable) -> Callable:
-    """Copy of ``pytorch_lightning.trainer.connectors.env_vars_connector._defaults_from_env_vars``. Required to fix
-    build error in readthedocs."""
+    """Copy of ``pytorch_lightning.trainer.connectors.env_vars_connector._defaults_from_env_vars``.
+
+    Required to fix build error in readthedocs.
+    """
 
     @wraps(fn)
     def insert_env_defaults(self, *args, **kwargs):
@@ -57,7 +60,7 @@ def _defaults_from_env_vars(fn: Callable) -> Callable:
             # parse only the argument names
             cls_arg_names = [arg[0] for arg in get_init_arguments_and_types(cls)]
             # convert args to kwargs
-            kwargs.update({k: v for k, v in zip(cls_arg_names, args)})
+            kwargs.update(dict(zip(cls_arg_names, args)))
         env_variables = vars(parse_env_variables(cls))
         # update the kwargs by env variables
         kwargs = dict(list(env_variables.items()) + list(kwargs.items()))
@@ -69,9 +72,8 @@ def _defaults_from_env_vars(fn: Callable) -> Callable:
 
 
 class Trainer(PlTrainer):
-
     @_defaults_from_env_vars
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, serve_sanity_check: bool = False, **kwargs):
         if flash._IS_TESTING:
             if torch.cuda.is_available():
                 kwargs["gpus"] = 1
@@ -84,6 +86,21 @@ class Trainer(PlTrainer):
                 kwargs["fast_dev_run"] = True
         super().__init__(*args, **kwargs)
 
+        self.serve_sanity_check = serve_sanity_check
+
+    def _run_sanity_check(self, ref_model):
+        if hasattr(super(), "_run_sanity_check"):
+            super()._run_sanity_check(ref_model)
+
+        self.run_sanity_check(ref_model)
+
+    def run_sanity_check(self, ref_model):
+        if hasattr(super(), "run_sanity_check"):
+            super().run_sanity_check(ref_model)
+
+        if self.serve_sanity_check and ref_model.is_servable and _SERVE_AVAILABLE:
+            ref_model.run_serve_sanity_check()
+
     def fit(
         self,
         model: LightningModule,
@@ -148,9 +165,7 @@ class Trainer(PlTrainer):
         return super().fit(model, train_dataloader, val_dataloaders, datamodule)
 
     def _resolve_callbacks(self, model, strategy):
-        """
-        This function is used to select the `BaseFinetuning` to be used for finetuning.
-        """
+        """This function is used to select the `BaseFinetuning` to be used for finetuning."""
         if strategy is not None and not isinstance(strategy, (str, BaseFinetuning)):
             raise MisconfigurationException(
                 "strategy should be a ``pytorch_lightning.callbacks.BaseFinetuning``"
@@ -170,7 +185,8 @@ class Trainer(PlTrainer):
                 if strategy is not None:
                     rank_zero_warn(
                         "The model contains a default finetune callback. The provided {strategy} will be overriden.\n"
-                        " HINT: Provide a `BaseFinetuning` callback as strategy to make it prioritized. ", UserWarning
+                        " HINT: Provide a `BaseFinetuning` callback as strategy to make it prioritized. ",
+                        UserWarning,
                     )
                 callback = model_callback
             else:
@@ -180,26 +196,72 @@ class Trainer(PlTrainer):
 
     @staticmethod
     def _merge_callbacks(old_callbacks: List, new_callbacks: List) -> List:
-        """
-        This function keeps only 1 instance of each callback type,
-        extending new_callbacks with old_callbacks
-        """
+        """This function keeps only 1 instance of each callback type, extending new_callbacks with
+        old_callbacks."""
         if len(new_callbacks) == 0:
             return old_callbacks
-        new_callbacks_types = set(type(c) for c in new_callbacks)
-        old_callbacks_types = set(type(c) for c in old_callbacks)
+        new_callbacks_types = {type(c) for c in new_callbacks}
+        old_callbacks_types = {type(c) for c in old_callbacks}
         override_types = new_callbacks_types.intersection(old_callbacks_types)
         new_callbacks.extend(c for c in old_callbacks if type(c) not in override_types)
         return new_callbacks
 
     @classmethod
     def add_argparse_args(cls, *args, **kwargs) -> ArgumentParser:
+        """See :func:`pytorch_lightning.utilities.argparse.add_argparse_args`."""
         # the lightning trainer implementation does not support subclasses.
         # context: https://github.com/PyTorchLightning/lightning-flash/issues/342#issuecomment-848892447
         return add_argparse_args(PlTrainer, *args, **kwargs)
 
     @classmethod
-    def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs) -> 'Trainer':
+    def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs) -> "Trainer":
+        """Modified version of :func:`pytorch_lightning.utilities.argparse.from_argparse_args` which populates
+        ``valid_kwargs`` from :class:`pytorch_lightning.Trainer`."""
         # the lightning trainer implementation does not support subclasses.
         # context: https://github.com/PyTorchLightning/lightning-flash/issues/342#issuecomment-848892447
-        return from_argparse_args(PlTrainer, args, **kwargs)
+        return from_argparse_args(Trainer, args, **kwargs)
+
+    def _parse_request_dataloader_args(self, args: Tuple, kwargs: Dict):
+        """Handles backwards compatibility for ``request_dataloader``.
+
+        Possible combinations:
+
+        legacy: (model, stage)
+        (stage, model)
+        (stage, model=model)
+        """
+        model, stage, is_legacy = None, None, False
+        if len(args) == 2:
+            # Check for legacy arguments: (model, stage)
+            if isinstance(args[0], LightningModule):
+                is_legacy = True
+                model, stage = args
+            else:  # (stage, model)
+                stage, model = args
+        else:
+            stage = kwargs.get("stage", args[0])
+            model = kwargs.get("model")
+        return model, stage, is_legacy
+
+    def request_dataloader(
+        self,
+        *args,
+        **kwargs,
+    ) -> Union[DataLoader, List[DataLoader]]:
+        """Handles downloading data in the GPU or TPU case.
+
+        Returns:
+            The dataloader
+        """
+        model, stage, is_legacy = self._parse_request_dataloader_args(args, kwargs)
+        if is_legacy:
+            self.call_hook(f"on_{stage}_dataloader")
+            dataloader = getattr(model, f"{stage}_dataloader")()
+        else:
+            hook = f"{stage.dataloader_prefix}_dataloader"
+            self.call_hook("on_" + hook, pl_module=model)
+            dataloader = self.call_hook(hook, pl_module=model)
+        if isinstance(dataloader, tuple):
+            dataloader = list(dataloader)
+        self.accelerator.barrier("get_dataloaders")
+        return dataloader
