@@ -12,8 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import os
-from dataclasses import dataclass
-from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Tuple, TYPE_CHECKING, Union
 
 import numpy as np
 import torch
@@ -29,62 +28,67 @@ from flash.core.data.data_module import DataModule
 from flash.core.data.data_source import (
     DefaultDataKeys,
     DefaultDataSources,
+    FiftyOneDataSource,
     ImageLabelsMap,
     NumpyDataSource,
     PathsDataSource,
-    SEQUENCE_DATA_TYPE,
     TensorDataSource,
 )
-from flash.core.data.process import Preprocess
-from flash.core.utilities.imports import _IMAGE_AVAILABLE, _MATPLOTLIB_AVAILABLE
+from flash.core.data.process import Deserializer, Preprocess
+from flash.core.utilities.imports import (
+    _FIFTYONE_AVAILABLE,
+    _MATPLOTLIB_AVAILABLE,
+    _TORCHVISION_AVAILABLE,
+    Image,
+    lazy_import,
+    requires,
+)
+from flash.image.data import ImageDeserializer, IMG_EXTENSIONS
 from flash.image.segmentation.serialization import SegmentationLabels
-from flash.image.segmentation.transforms import default_transforms, train_default_transforms
+from flash.image.segmentation.transforms import default_transforms, predict_default_transforms, train_default_transforms
+
+SampleCollection = None
+if _FIFTYONE_AVAILABLE:
+    fo = lazy_import("fiftyone")
+    if TYPE_CHECKING:
+        from fiftyone.core.collections import SampleCollection
+else:
+    fo = None
 
 if _MATPLOTLIB_AVAILABLE:
     import matplotlib.pyplot as plt
 else:
     plt = None
 
-if _IMAGE_AVAILABLE:
+if _TORCHVISION_AVAILABLE:
     import torchvision
-    from PIL import Image
-    from torchvision.datasets.folder import has_file_allowed_extension, IMG_EXTENSIONS
-
-else:
-
-    class Image:
-        Image = None
-
-    IMG_EXTENSIONS = None
+    import torchvision.transforms.functional as FT
+    from torchvision.datasets.folder import default_loader, has_file_allowed_extension
 
 
 class SemanticSegmentationNumpyDataSource(NumpyDataSource):
-
     def load_sample(self, sample: Dict[str, Any], dataset: Optional[Any] = None) -> Dict[str, Any]:
         img = torch.from_numpy(sample[DefaultDataKeys.INPUT]).float()
         sample[DefaultDataKeys.INPUT] = img
-        sample[DefaultDataKeys.METADATA] = img.shape
+        sample[DefaultDataKeys.METADATA] = {"size": img.shape}
         return sample
 
 
 class SemanticSegmentationTensorDataSource(TensorDataSource):
-
     def load_sample(self, sample: Dict[str, Any], dataset: Optional[Any] = None) -> Dict[str, Any]:
         img = sample[DefaultDataKeys.INPUT].float()
         sample[DefaultDataKeys.INPUT] = img
-        sample[DefaultDataKeys.METADATA] = img.shape
+        sample[DefaultDataKeys.METADATA] = {"size": img.shape}
         return sample
 
 
 class SemanticSegmentationPathsDataSource(PathsDataSource):
-
     def __init__(self):
-        if not _IMAGE_AVAILABLE:
-            raise ModuleNotFoundError("Please, pip install 'lightning-flash[image]'")
         super().__init__(IMG_EXTENSIONS)
 
-    def load_data(self, data: Union[Tuple[str, str], Tuple[List[str], List[str]]],
-                  dataset: BaseAutoDataset) -> Sequence[Mapping[str, Any]]:
+    def load_data(
+        self, data: Union[Tuple[str, str], Tuple[List[str], List[str]]], dataset: BaseAutoDataset
+    ) -> Sequence[Mapping[str, Any]]:
         input_data, target_data = data
 
         if self.isdir(input_data) and self.isdir(target_data):
@@ -115,13 +119,15 @@ class SemanticSegmentationPathsDataSource(PathsDataSource):
 
         data = filter(
             lambda sample: (
-                has_file_allowed_extension(sample[0], self.extensions) and
-                has_file_allowed_extension(sample[1], self.extensions)
+                has_file_allowed_extension(sample[0], self.extensions)
+                and has_file_allowed_extension(sample[1], self.extensions)
             ),
             zip(input_data, target_data),
         )
 
-        return [{DefaultDataKeys.INPUT: input, DefaultDataKeys.TARGET: target} for input, target in data]
+        data = [{DefaultDataKeys.INPUT: input, DefaultDataKeys.TARGET: target} for input, target in data]
+
+        return data
 
     def predict_load_data(self, data: Union[str, List[str]]):
         return super().predict_load_data(data)
@@ -132,35 +138,96 @@ class SemanticSegmentationPathsDataSource(PathsDataSource):
         img_labels_path = sample[DefaultDataKeys.TARGET]
 
         # load images directly to torch tensors
-        img: torch.Tensor = torchvision.io.read_image(img_path)  # CxHxW
+        img: torch.Tensor = FT.to_tensor(default_loader(img_path))  # CxHxW
         img_labels: torch.Tensor = torchvision.io.read_image(img_labels_path)  # CxHxW
         img_labels = img_labels[0]  # HxW
 
-        return {
-            DefaultDataKeys.INPUT: img.float(),
-            DefaultDataKeys.TARGET: img_labels.float(),
-            DefaultDataKeys.METADATA: img.shape,
+        sample[DefaultDataKeys.INPUT] = img.float()
+        sample[DefaultDataKeys.TARGET] = img_labels.float()
+        sample[DefaultDataKeys.METADATA] = {
+            "filepath": img_path,
+            "size": img.shape,
         }
+        return sample
 
-    def predict_load_sample(self, sample: Mapping[str, Any]) -> Mapping[str, Any]:
-        img = torchvision.io.read_image(sample[DefaultDataKeys.INPUT]).float()
-        return {
-            DefaultDataKeys.INPUT: img,
-            DefaultDataKeys.METADATA: img.shape,
+    @staticmethod
+    def predict_load_sample(sample: Mapping[str, Any]) -> Mapping[str, Any]:
+        img_path = sample[DefaultDataKeys.INPUT]
+        img = FT.to_tensor(default_loader(img_path)).float()
+
+        sample[DefaultDataKeys.INPUT] = img
+        sample[DefaultDataKeys.METADATA] = {
+            "filepath": img_path,
+            "size": img.shape,
         }
+        return sample
+
+
+class SemanticSegmentationFiftyOneDataSource(FiftyOneDataSource):
+    def __init__(self, label_field: str = "ground_truth"):
+        super().__init__(label_field=label_field)
+        self._fo_dataset_name = None
+
+    @property
+    def label_cls(self):
+        return fo.Segmentation
+
+    def load_data(self, data: SampleCollection, dataset: Optional[Any] = None) -> Sequence[Mapping[str, Any]]:
+        self._validate(data)
+
+        self._fo_dataset_name = data.name
+        return [{DefaultDataKeys.INPUT: f} for f in data.values("filepath")]
+
+    def load_sample(self, sample: Mapping[str, str]) -> Mapping[str, Union[torch.Tensor, torch.Size]]:
+        _fo_dataset = fo.load_dataset(self._fo_dataset_name)
+
+        img_path = sample[DefaultDataKeys.INPUT]
+        fo_sample = _fo_dataset[img_path]
+
+        img: torch.Tensor = FT.to_tensor(default_loader(img_path))  # CxHxW
+        img_labels: torch.Tensor = torch.from_numpy(fo_sample[self.label_field].mask)  # HxW
+
+        sample[DefaultDataKeys.INPUT] = img.float()
+        sample[DefaultDataKeys.TARGET] = img_labels.float()
+        sample[DefaultDataKeys.METADATA] = {
+            "filepath": img_path,
+            "size": img.shape,
+        }
+        return sample
+
+    @staticmethod
+    def predict_load_sample(sample: Mapping[str, Any]) -> Mapping[str, Any]:
+        img_path = sample[DefaultDataKeys.INPUT]
+        img = FT.to_tensor(default_loader(img_path)).float()
+
+        sample[DefaultDataKeys.INPUT] = img
+        sample[DefaultDataKeys.METADATA] = {
+            "filepath": img_path,
+            "size": img.shape,
+        }
+        return sample
+
+
+class SemanticSegmentationDeserializer(ImageDeserializer):
+    def deserialize(self, data: str) -> torch.Tensor:
+        result = super().deserialize(data)
+        result[DefaultDataKeys.INPUT] = FT.to_tensor(result[DefaultDataKeys.INPUT])
+        result[DefaultDataKeys.METADATA] = {"size": result[DefaultDataKeys.INPUT].shape}
+        return result
 
 
 class SemanticSegmentationPreprocess(Preprocess):
-
     def __init__(
         self,
         train_transform: Optional[Dict[str, Callable]] = None,
         val_transform: Optional[Dict[str, Callable]] = None,
         test_transform: Optional[Dict[str, Callable]] = None,
         predict_transform: Optional[Dict[str, Callable]] = None,
-        image_size: Tuple[int, int] = (196, 196),
+        image_size: Tuple[int, int] = (128, 128),
+        deserializer: Optional["Deserializer"] = None,
         num_classes: int = None,
         labels_map: Dict[int, Tuple[int, int, int]] = None,
+        **data_source_kwargs: Any,
     ) -> None:
         """Preprocess pipeline for semantic segmentation tasks.
 
@@ -170,9 +237,8 @@ class SemanticSegmentationPreprocess(Preprocess):
             test_transform: Dictionary with the set of transforms to apply during testing.
             predict_transform: Dictionary with the set of transforms to apply during prediction.
             image_size: A tuple with the expected output image size.
+            **data_source_kwargs: Additional arguments passed on to the data source constructors.
         """
-        if not _IMAGE_AVAILABLE:
-            raise ModuleNotFoundError("Please, pip install 'lightning-flash[image]'")
         self.image_size = image_size
         self.num_classes = num_classes
         if num_classes:
@@ -184,11 +250,13 @@ class SemanticSegmentationPreprocess(Preprocess):
             test_transform=test_transform,
             predict_transform=predict_transform,
             data_sources={
+                DefaultDataSources.FIFTYONE: SemanticSegmentationFiftyOneDataSource(**data_source_kwargs),
                 DefaultDataSources.FILES: SemanticSegmentationPathsDataSource(),
                 DefaultDataSources.FOLDERS: SemanticSegmentationPathsDataSource(),
                 DefaultDataSources.TENSORS: SemanticSegmentationTensorDataSource(),
                 DefaultDataSources.NUMPY: SemanticSegmentationNumpyDataSource(),
             },
+            deserializer=deserializer or SemanticSegmentationDeserializer(),
             default_data_source=DefaultDataSources.FILES,
         )
 
@@ -199,9 +267,10 @@ class SemanticSegmentationPreprocess(Preprocess):
 
     def get_state_dict(self) -> Dict[str, Any]:
         return {
-            **self.transforms, "image_size": self.image_size,
+            **self.transforms,
+            "image_size": self.image_size,
             "num_classes": self.num_classes,
-            "labels_map": self.labels_map
+            "labels_map": self.labels_map,
         }
 
     @classmethod
@@ -214,6 +283,9 @@ class SemanticSegmentationPreprocess(Preprocess):
     def train_default_transforms(self) -> Optional[Dict[str, Callable]]:
         return train_default_transforms(self.image_size)
 
+    def predict_default_transforms(self) -> Optional[Dict[str, Callable]]:
+        return predict_default_transforms(self.image_size)
+
 
 class SemanticSegmentationData(DataModule):
     """Data module for semantic segmentation tasks."""
@@ -223,7 +295,7 @@ class SemanticSegmentationData(DataModule):
     @staticmethod
     def configure_data_fetcher(
         labels_map: Optional[Dict[int, Tuple[int, int, int]]] = None
-    ) -> 'SegmentationMatplotlibVisualization':
+    ) -> "SegmentationMatplotlibVisualization":
         return SegmentationMatplotlibVisualization(labels_map=labels_map)
 
     def set_block_viz_window(self, value: bool) -> None:
@@ -248,22 +320,23 @@ class SemanticSegmentationData(DataModule):
         batch_size: int = 4,
         num_workers: Optional[int] = None,
         **preprocess_kwargs: Any,
-    ) -> 'DataModule':
+    ) -> "DataModule":
 
-        if 'num_classes' not in preprocess_kwargs:
+        if "num_classes" not in preprocess_kwargs:
             raise MisconfigurationException("`num_classes` should be provided during instantiation.")
 
         num_classes = preprocess_kwargs["num_classes"]
 
-        labels_map = getattr(preprocess_kwargs, "labels_map",
-                             None) or SegmentationLabels.create_random_labels_map(num_classes)
+        labels_map = getattr(preprocess_kwargs, "labels_map", None) or SegmentationLabels.create_random_labels_map(
+            num_classes
+        )
 
         data_fetcher = data_fetcher or cls.configure_data_fetcher(labels_map)
 
         if flash._IS_TESTING:
             data_fetcher.block_viz_window = True
 
-        dm = super(SemanticSegmentationData, cls).from_data_source(
+        dm = super().from_data_source(
             data_source=data_source,
             train_data=train_data,
             val_data=val_data,
@@ -278,10 +351,11 @@ class SemanticSegmentationData(DataModule):
             val_split=val_split,
             batch_size=batch_size,
             num_workers=num_workers,
-            **preprocess_kwargs
+            **preprocess_kwargs,
         )
 
-        dm.train_dataset.num_classes = num_classes
+        if dm.train_dataset is not None:
+            dm.train_dataset.num_classes = num_classes
         return dm
 
     @classmethod
@@ -306,7 +380,7 @@ class SemanticSegmentationData(DataModule):
         num_classes: Optional[int] = None,
         labels_map: Dict[int, Tuple[int, int, int]] = None,
         **preprocess_kwargs,
-    ) -> 'DataModule':
+    ) -> "DataModule":
         """Creates a :class:`~flash.image.segmentation.data.SemanticSegmentationData` object from the given data
         folders and corresponding target folders.
 
@@ -374,8 +448,7 @@ class SemanticSegmentationData(DataModule):
 
 
 class SegmentationMatplotlibVisualization(BaseVisualization):
-    """Process and show the image batch and its associated label using matplotlib.
-    """
+    """Process and show the image batch and its associated label using matplotlib."""
 
     def __init__(self, labels_map: Dict[int, Tuple[int, int, int]]):
         super().__init__()
@@ -385,6 +458,7 @@ class SegmentationMatplotlibVisualization(BaseVisualization):
         self.labels_map: Dict[int, Tuple[int, int, int]] = labels_map
 
     @staticmethod
+    @requires("image")
     def _to_numpy(img: Union[torch.Tensor, Image.Image]) -> np.ndarray:
         out: np.ndarray
         if isinstance(img, Image.Image):
@@ -395,14 +469,12 @@ class SegmentationMatplotlibVisualization(BaseVisualization):
             raise TypeError(f"Unknown image type. Got: {type(img)}.")
         return out
 
+    @requires("matplotlib")
     def _show_images_and_labels(self, data: List[Any], num_samples: int, title: str):
         # define the image grid
         cols: int = min(num_samples, self.max_cols)
         rows: int = num_samples // cols
 
-        if not _MATPLOTLIB_AVAILABLE:
-            raise MisconfigurationException("You need matplotlib to visualise. Please, pip install matplotlib")
-
         # create figure and set title
         fig, axs = plt.subplots(rows, cols)
         fig.suptitle(title)
@@ -425,7 +497,7 @@ class SegmentationMatplotlibVisualization(BaseVisualization):
             img_vis = np.hstack((image_vis, label_vis))
             # send to visualiser
             ax.imshow(img_vis)
-            ax.axis('off')
+            ax.axis("off")
         plt.show(block=self.block_viz_window)
 
     def show_load_sample(self, samples: List[Any], running_stage: RunningStage):
