@@ -17,17 +17,19 @@ import torch
 import torch.nn as nn
 
 from flash.core.data.data_source import DefaultDataKeys
-from flash.core.utilities.imports import _IMAGE_AVAILABLE
+from flash.core.utilities.imports import _KORNIA_AVAILABLE, _TORCHVISION_AVAILABLE
 
-if _IMAGE_AVAILABLE:
+if _KORNIA_AVAILABLE:
     import kornia as K
+
+if _TORCHVISION_AVAILABLE:
     from torchvision.transforms import Compose
 
     from flash.core.data.transforms import ApplyToKeys, kornia_collate, KorniaParallelTransforms, merge_transforms
 
 
 def prepare_target(tensor: torch.Tensor) -> torch.Tensor:
-    """ Convert the target mask to long and remove the channel dimension. """
+    """Convert the target mask to long and remove the channel dimension."""
     return tensor.long().squeeze(1)
 
 
@@ -38,7 +40,7 @@ def default_transforms(image_size: Tuple[int, int]) -> Dict[str, Callable]:
         "post_tensor_transform": nn.Sequential(
             ApplyToKeys(
                 [DefaultDataKeys.INPUT, DefaultDataKeys.TARGET],
-                KorniaParallelTransforms(K.geometry.Resize(image_size, interpolation='nearest')),
+                KorniaParallelTransforms(K.geometry.Resize(image_size, interpolation="nearest")),
             ),
         ),
         "collate": Compose([kornia_collate, ApplyToKeys(DefaultDataKeys.TARGET, prepare_target)]),
@@ -46,14 +48,29 @@ def default_transforms(image_size: Tuple[int, int]) -> Dict[str, Callable]:
 
 
 def train_default_transforms(image_size: Tuple[int, int]) -> Dict[str, Callable]:
-    """During training, we apply the default transforms with additional ``RandomHorizontalFlip`` and ``ColorJitter``."""
+    """During training, we apply the default transforms with additional ``RandomHorizontalFlip`` and
+    ``ColorJitter``."""
     return merge_transforms(
-        default_transforms(image_size), {
+        default_transforms(image_size),
+        {
             "post_tensor_transform": nn.Sequential(
                 ApplyToKeys(
                     [DefaultDataKeys.INPUT, DefaultDataKeys.TARGET],
                     KorniaParallelTransforms(K.augmentation.RandomHorizontalFlip(p=0.5)),
                 ),
             ),
-        }
+        },
     )
+
+
+def predict_default_transforms(image_size: Tuple[int, int]) -> Dict[str, Callable]:
+    """During predict, we apply the default transforms only on DefaultDataKeys.INPUT."""
+    return {
+        "post_tensor_transform": nn.Sequential(
+            ApplyToKeys(
+                DefaultDataKeys.INPUT,
+                K.geometry.Resize(image_size, interpolation="nearest"),
+            ),
+        ),
+        "collate": kornia_collate,
+    }
