@@ -21,12 +21,12 @@ from flash.core.data.data_source import DefaultDataKeys
 from flash.core.data.process import Serializer
 from flash.core.model import Task
 from flash.core.registry import FlashRegistry
-from flash.core.utilities.imports import _IMAGE_STLYE_TRANSFER
+from flash.core.utilities.imports import _IMAGE_AVAILABLE
 from flash.image.style_transfer import STYLE_TRANSFER_BACKBONES
 
-if _IMAGE_STLYE_TRANSFER:
+if _IMAGE_AVAILABLE:
     import pystiche.demo
-    from pystiche import enc, loss, ops
+    from pystiche import enc, loss
     from pystiche.image import read_image
 else:
 
@@ -34,12 +34,9 @@ else:
         Encoder = None
         MultiLayerEncoder = None
 
-    class ops:
-        EncodingComparisonOperator = None
-        FeatureReconstructionOperator = None
-        MultiLayerEncodingOperator = None
-
     class loss:
+        class GramLoss:
+            pass
 
         class PerceptualLoss:
             pass
@@ -51,13 +48,8 @@ __all__ = ["StyleTransfer"]
 
 
 class StyleTransfer(Task):
-    """Task that transfer the style from an image onto another.
-
-    Example::
-
-        from flash.image.style_transfer import StyleTransfer
-
-        model = StyleTransfer(image_style)
+    """``StyleTransfer`` is a :class:`~flash.Task` for transferring the style from one image onto another. For more
+    details, see :ref:`style_transfer`.
 
     Args:
         style_image: Image or path to an image to derive the style from.
@@ -76,6 +68,8 @@ class StyleTransfer(Task):
 
     backbones: FlashRegistry = STYLE_TRANSFER_BACKBONES
 
+    required_extras: str = "image"
+
     def __init__(
         self,
         style_image: Optional[Union[str, torch.Tensor]] = None,
@@ -83,7 +77,7 @@ class StyleTransfer(Task):
         backbone: str = "vgg16",
         content_layer: str = "relu2_2",
         content_weight: float = 1e5,
-        style_layers: Union[Sequence[str], str] = ("relu1_2", "relu2_2", "relu3_3", "relu4_3"),
+        style_layers: Union[Sequence[str], str] = ["relu1_2", "relu2_2", "relu3_3", "relu4_3"],
         style_weight: float = 1e10,
         optimizer: Union[Type[torch.optim.Optimizer], torch.optim.Optimizer] = torch.optim.Adam,
         optimizer_kwargs: Optional[Dict[str, Any]] = None,
@@ -92,10 +86,6 @@ class StyleTransfer(Task):
         learning_rate: float = 1e-3,
         serializer: Optional[Union[Serializer, Mapping[str, Serializer]]] = None,
     ):
-
-        if not _IMAGE_STLYE_TRANSFER:
-            raise ModuleNotFoundError("Please, pip install 'lightning-flash[image_style_transfer]'")
-
         self.save_hyperparameters(ignore="style_image")
 
         if style_image is None:
@@ -107,7 +97,7 @@ class StyleTransfer(Task):
             model = pystiche.demo.transformer()
 
         if not isinstance(style_layers, (List, Tuple)):
-            style_layers = (style_layers, )
+            style_layers = (style_layers,)
 
         perceptual_loss = self._get_perceptual_loss(
             backbone=backbone,
@@ -131,16 +121,16 @@ class StyleTransfer(Task):
 
         self.perceptual_loss = perceptual_loss
 
-    def default_style_image(self) -> torch.Tensor:
+    @staticmethod
+    def default_style_image() -> torch.Tensor:
         return pystiche.demo.images()["paint"].read(size=256)
 
     @staticmethod
-    def _modified_gram_loss(encoder: enc.Encoder, *, score_weight: float) -> ops.EncodingComparisonOperator:
+    def _modified_gram_loss(encoder: enc.Encoder, *, score_weight: float) -> loss.GramLoss:
         # The official PyTorch examples as well as the reference implementation of the original author contain an
         # oversight: they normalize the representation twice by the number of channels. To be compatible with them, we
         # do the same here.
-        class GramOperator(ops.GramOperator):
-
+        class GramOperator(loss.GramLoss):
             def enc_to_repr(self, enc: torch.Tensor) -> torch.Tensor:
                 repr = super().enc_to_repr(enc)
                 num_channels = repr.size()[1]
@@ -158,10 +148,8 @@ class StyleTransfer(Task):
         style_weight: float,
     ) -> loss.PerceptualLoss:
         mle, _ = cast(enc.MultiLayerEncoder, self.backbones.get(backbone)())
-        content_loss = ops.FeatureReconstructionOperator(
-            mle.extract_encoder(content_layer), score_weight=content_weight
-        )
-        style_loss = ops.MultiLayerEncodingOperator(
+        content_loss = loss.FeatureReconstructionLoss(mle.extract_encoder(content_layer), score_weight=content_weight)
+        style_loss = loss.MultiLayerEncodingLoss(
             mle,
             style_layers,
             lambda encoder, layer_weight: self._modified_gram_loss(encoder, score_weight=layer_weight),
@@ -182,6 +170,6 @@ class StyleTransfer(Task):
     def test_step(self, batch: Any, batch_idx: int) -> NoReturn:
         raise_not_supported("test")
 
-    def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: int) -> Any:
+    def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: int = 0) -> Any:
         input_image = batch[DefaultDataKeys.INPUT]
         return self(input_image)
