@@ -18,7 +18,7 @@ import torchmetrics
 from torch import nn
 from torch.optim.lr_scheduler import _LRScheduler
 
-from flash.core.classification import ClassificationTask
+from flash.core.classification import ClassificationTask, Labels
 from flash.core.data.data_source import DefaultDataKeys
 from flash.core.data.process import Serializer
 from flash.core.registry import FlashRegistry
@@ -26,8 +26,8 @@ from flash.template.classification.backbones import TEMPLATE_BACKBONES
 
 
 class TemplateSKLearnClassifier(ClassificationTask):
-    """The ``TemplateSKLearnClassifier`` is a :class:`~flash.core.classification.ClassificationTask` that classifies
-    tabular data from scikit-learn.
+    """The ``TemplateSKLearnClassifier`` is a :class:`~flash.core.classification.ClassificationTask` that
+    classifies tabular data from scikit-learn.
 
     Args:
         num_features: The number of features (elements) in the input data.
@@ -75,7 +75,7 @@ class TemplateSKLearnClassifier(ClassificationTask):
             metrics=metrics,
             learning_rate=learning_rate,
             multi_label=multi_label,
-            serializer=serializer,
+            serializer=serializer or Labels(),
         )
 
         self.save_hyperparameters()
@@ -112,9 +112,9 @@ class TemplateSKLearnClassifier(ClassificationTask):
         return super().test_step(batch, batch_idx)
 
     def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: int = 0) -> Any:
-        """For the predict step, we just extract the :attr:`~flash.core.data.data_source.DefaultDataKeys.INPUT` key from
-        the input and forward it to the :meth:`~flash.core.model.Task.predict_step`."""
-        batch = (batch[DefaultDataKeys.INPUT])
+        """For the predict step, we just extract the :attr:`~flash.core.data.data_source.DefaultDataKeys.INPUT` key
+        from the input and forward it to the :meth:`~flash.core.model.Task.predict_step`."""
+        batch = batch[DefaultDataKeys.INPUT]
         return super().predict_step(batch, batch_idx, dataloader_idx=dataloader_idx)
 
     def forward(self, x) -> torch.Tensor:
